@@ -38,41 +38,48 @@ namespace JSC {
 class Structure;
 
 typedef uint32_t SpeculatedType;
-static const SpeculatedType SpecNone              = 0x00000000; // We don't know anything yet.
-static const SpeculatedType SpecFinalObject       = 0x00000001; // It's definitely a JSFinalObject.
-static const SpeculatedType SpecArray             = 0x00000002; // It's definitely a JSArray.
-static const SpeculatedType SpecFunction          = 0x00000008; // It's definitely a JSFunction or one of its subclasses.
-static const SpeculatedType SpecInt8Array         = 0x00000010; // It's definitely an Int8Array or one of its subclasses.
-static const SpeculatedType SpecInt16Array        = 0x00000020; // It's definitely an Int16Array or one of its subclasses.
-static const SpeculatedType SpecInt32Array        = 0x00000040; // It's definitely an Int32Array or one of its subclasses.
-static const SpeculatedType SpecUint8Array        = 0x00000080; // It's definitely an Uint8Array or one of its subclasses.
-static const SpeculatedType SpecUint8ClampedArray = 0x00000100; // It's definitely an Uint8ClampedArray or one of its subclasses.
-static const SpeculatedType SpecUint16Array       = 0x00000200; // It's definitely an Uint16Array or one of its subclasses.
-static const SpeculatedType SpecUint32Array       = 0x00000400; // It's definitely an Uint32Array or one of its subclasses.
-static const SpeculatedType SpecFloat32Array      = 0x00000800; // It's definitely an Uint16Array or one of its subclasses.
-static const SpeculatedType SpecFloat64Array      = 0x00001000; // It's definitely an Uint16Array or one of its subclasses.
-static const SpeculatedType SpecTypedArrayView    = SpecInt8Array | SpecInt16Array | SpecInt32Array | SpecUint8Array | SpecUint8ClampedArray | SpecUint16Array | SpecUint32Array | SpecFloat32Array | SpecFloat64Array;
-static const SpeculatedType SpecArguments         = 0x00002000; // It's definitely an Arguments object.
-static const SpeculatedType SpecStringObject      = 0x00004000; // It's definitely a StringObject.
-static const SpeculatedType SpecObjectOther       = 0x00008000; // It's definitely an object but not JSFinalObject, JSArray, or JSFunction.
-static const SpeculatedType SpecObject            = 0x0000ffff; // Bitmask used for testing for any kind of object prediction.
-static const SpeculatedType SpecStringIdent       = 0x00010000; // It's definitely a JSString, and it's an identifier.
-static const SpeculatedType SpecStringVar         = 0x00020000; // It's definitely a JSString, and it's not an identifier.
-static const SpeculatedType SpecString            = 0x00030000; // It's definitely a JSString.
-static const SpeculatedType SpecCellOther         = 0x00040000; // It's definitely a JSCell but not a subclass of JSObject and definitely not a JSString.
-static const SpeculatedType SpecCell              = 0x0007ffff; // It's definitely a JSCell.
-static const SpeculatedType SpecInt32             = 0x00800000; // It's definitely an Int32.
-static const SpeculatedType SpecDoubleReal        = 0x01000000; // It's definitely a non-NaN double.
-static const SpeculatedType SpecDoubleNaN         = 0x02000000; // It's definitely a NaN.
-static const SpeculatedType SpecDouble            = 0x03000000; // It's either a non-NaN or a NaN double.
-static const SpeculatedType SpecRealNumber        = 0x01800000; // It's either an Int32 or a DoubleReal.
-static const SpeculatedType SpecNumber            = 0x03800000; // It's either an Int32 or a Double.
-static const SpeculatedType SpecBoolean           = 0x04000000; // It's definitely a Boolean.
-static const SpeculatedType SpecOther             = 0x08000000; // It's definitely none of the above.
-static const SpeculatedType SpecTop               = 0x0fffffff; // It can be any of the above.
-static const SpeculatedType SpecEmpty             = 0x10000000; // It's definitely an empty value marker.
-static const SpeculatedType SpecEmptyOrTop        = 0x1fffffff; // It can be any of the above.
-static const SpeculatedType FixedIndexedStorageMask    = SpecInt8Array | SpecInt16Array | SpecInt32Array | SpecUint8Array | SpecUint8ClampedArray | SpecUint16Array | SpecUint32Array | SpecFloat32Array | SpecFloat64Array;
+static const SpeculatedType SpecNone               = 0x00000000; // We don't know anything yet.
+static const SpeculatedType SpecFinalObject        = 0x00000001; // It's definitely a JSFinalObject.
+static const SpeculatedType SpecArray              = 0x00000002; // It's definitely a JSArray.
+static const SpeculatedType SpecFunction           = 0x00000008; // It's definitely a JSFunction or one of its subclasses.
+static const SpeculatedType SpecInt8Array          = 0x00000010; // It's definitely an Int8Array or one of its subclasses.
+static const SpeculatedType SpecInt16Array         = 0x00000020; // It's definitely an Int16Array or one of its subclasses.
+static const SpeculatedType SpecInt32Array         = 0x00000040; // It's definitely an Int32Array or one of its subclasses.
+static const SpeculatedType SpecUint8Array         = 0x00000080; // It's definitely an Uint8Array or one of its subclasses.
+static const SpeculatedType SpecUint8ClampedArray  = 0x00000100; // It's definitely an Uint8ClampedArray or one of its subclasses.
+static const SpeculatedType SpecUint16Array        = 0x00000200; // It's definitely an Uint16Array or one of its subclasses.
+static const SpeculatedType SpecUint32Array        = 0x00000400; // It's definitely an Uint32Array or one of its subclasses.
+static const SpeculatedType SpecFloat32Array       = 0x00000800; // It's definitely an Uint16Array or one of its subclasses.
+static const SpeculatedType SpecFloat64Array       = 0x00001000; // It's definitely an Uint16Array or one of its subclasses.
+static const SpeculatedType SpecTypedArrayView     = SpecInt8Array | SpecInt16Array | SpecInt32Array | SpecUint8Array | SpecUint8ClampedArray | SpecUint16Array | SpecUint32Array | SpecFloat32Array | SpecFloat64Array;
+static const SpeculatedType SpecArguments          = 0x00002000; // It's definitely an Arguments object.
+static const SpeculatedType SpecStringObject       = 0x00004000; // It's definitely a StringObject.
+static const SpeculatedType SpecObjectOther        = 0x00008000; // It's definitely an object but not JSFinalObject, JSArray, or JSFunction.
+static const SpeculatedType SpecObject             = 0x0000ffff; // Bitmask used for testing for any kind of object prediction.
+static const SpeculatedType SpecStringIdent        = 0x00010000; // It's definitely a JSString, and it's an identifier.
+static const SpeculatedType SpecStringVar          = 0x00020000; // It's definitely a JSString, and it's not an identifier.
+static const SpeculatedType SpecString             = 0x00030000; // It's definitely a JSString.
+static const SpeculatedType SpecCellOther          = 0x00040000; // It's definitely a JSCell but not a subclass of JSObject and definitely not a JSString.
+static const SpeculatedType SpecCell               = 0x0007ffff; // It's definitely a JSCell.
+static const SpeculatedType SpecInt32              = 0x00800000; // It's definitely an Int32.
+static const SpeculatedType SpecInt52              = 0x01000000; // It's definitely an Int52 and we intend it to unbox it.
+static const SpeculatedType SpecMachineInt         = 0x01800000; // It's something that we can do machine int arithmetic on.
+static const SpeculatedType SpecInt52AsDouble      = 0x02000000; // It's definitely an Int52 and it's inside a double.
+static const SpeculatedType SpecInteger            = 0x03800000; // It's definitely some kind of integer.
+static const SpeculatedType SpecNonIntAsDouble     = 0x04000000; // It's definitely not an Int52 but it's a real number and it's a double.
+static const SpeculatedType SpecDoubleReal         = 0x06000000; // It's definitely a non-NaN double.
+static const SpeculatedType SpecDoubleNaN          = 0x08000000; // It's definitely a NaN.
+static const SpeculatedType SpecDouble             = 0x0e000000; // It's either a non-NaN or a NaN double.
+static const SpeculatedType SpecBytecodeRealNumber = 0x06800000; // It's either an Int32 or a DoubleReal.
+static const SpeculatedType SpecFullRealNumber     = 0x07800000; // It's either an Int32 or a DoubleReal, or a Int52.
+static const SpeculatedType SpecBytecodeNumber     = 0x0e800000; // It's either an Int32 or a Double.
+static const SpeculatedType SpecFullNumber         = 0x0f800000; // It's either an Int32, Int52, or a Double.
+static const SpeculatedType SpecBoolean            = 0x10000000; // It's definitely a Boolean.
+static const SpeculatedType SpecOther              = 0x20000000; // It's definitely none of the above.
+static const SpeculatedType SpecHeapTop            = 0x3effffff; // It can be any of the above, except for SpecInt52.
+static const SpeculatedType SpecEmpty              = 0x40000000; // It's definitely an empty value marker.
+static const SpeculatedType SpecBytecodeTop        = 0x7effffff; // It can be any of the above, except for SpecInt52.
+static const SpeculatedType SpecFullTop            = 0x7fffffff; // It can be any of the above plus anything the DFG chooses.
 
 typedef bool (*SpeculatedTypeChecker)(SpeculatedType);
 
@@ -107,11 +114,6 @@ inline bool isFinalObjectOrOtherSpeculation(SpeculatedType value)
     return !!(value & (SpecFinalObject | SpecOther)) && !(value & ~(SpecFinalObject | SpecOther));
 }
 
-inline bool isFixedIndexedStorageObjectSpeculation(SpeculatedType value)
-{
-    return !!value && (value & FixedIndexedStorageMask) == value;
-}
-
 inline bool isStringIdentSpeculation(SpeculatedType value)
 {
     return value == SpecStringIdent;
@@ -240,7 +242,7 @@ inline bool isInt32Speculation(SpeculatedType value)
 
 inline bool isInt32SpeculationForArithmetic(SpeculatedType value)
 {
-    return !(value & SpecDouble);
+    return !(value & (SpecDouble | SpecInt52));
 }
 
 inline bool isInt32SpeculationExpectingDefined(SpeculatedType value)
@@ -248,9 +250,39 @@ inline bool isInt32SpeculationExpectingDefined(SpeculatedType value)
     return isInt32Speculation(value & ~SpecOther);
 }
 
+inline bool isInt52Speculation(SpeculatedType value)
+{
+    return value == SpecInt52;
+}
+
+inline bool isMachineIntSpeculation(SpeculatedType value)
+{
+    return !!value && (value & SpecMachineInt) == value;
+}
+
+inline bool isMachineIntSpeculationExpectingDefined(SpeculatedType value)
+{
+    return isMachineIntSpeculation(value & ~SpecOther);
+}
+
+inline bool isMachineIntSpeculationForArithmetic(SpeculatedType value)
+{
+    return !(value & SpecDouble);
+}
+
+inline bool isInt52AsDoubleSpeculation(SpeculatedType value)
+{
+    return value == SpecInt52AsDouble;
+}
+
+inline bool isIntegerSpeculation(SpeculatedType value)
+{
+    return !!value && (value & SpecInteger) == value;
+}
+
 inline bool isDoubleRealSpeculation(SpeculatedType value)
 {
-    return value == SpecDoubleReal;
+    return !!value && (value & SpecDoubleReal) == value;
 }
 
 inline bool isDoubleSpeculation(SpeculatedType value)
@@ -263,19 +295,34 @@ inline bool isDoubleSpeculationForArithmetic(SpeculatedType value)
     return !!(value & SpecDouble);
 }
 
-inline bool isRealNumberSpeculation(SpeculatedType value)
+inline bool isBytecodeRealNumberSpeculation(SpeculatedType value)
 {
-    return !!(value & SpecRealNumber) && !(value & ~SpecRealNumber);
+    return !!(value & SpecBytecodeRealNumber) && !(value & ~SpecBytecodeRealNumber);
 }
 
-inline bool isNumberSpeculation(SpeculatedType value)
+inline bool isFullRealNumberSpeculation(SpeculatedType value)
 {
-    return !!(value & SpecNumber) && !(value & ~SpecNumber);
+    return !!(value & SpecFullRealNumber) && !(value & ~SpecFullRealNumber);
 }
 
-inline bool isNumberSpeculationExpectingDefined(SpeculatedType value)
+inline bool isBytecodeNumberSpeculation(SpeculatedType value)
 {
-    return isNumberSpeculation(value & ~SpecOther);
+    return !!(value & SpecBytecodeNumber) && !(value & ~SpecBytecodeNumber);
+}
+
+inline bool isFullNumberSpeculation(SpeculatedType value)
+{
+    return !!(value & SpecFullNumber) && !(value & ~SpecFullNumber);
+}
+
+inline bool isBytecodeNumberSpeculationExpectingDefined(SpeculatedType value)
+{
+    return isBytecodeNumberSpeculation(value & ~SpecOther);
+}
+
+inline bool isFullNumberSpeculationExpectingDefined(SpeculatedType value)
+{
+    return isFullNumberSpeculation(value & ~SpecOther);
 }
 
 inline bool isBooleanSpeculation(SpeculatedType value)
