@@ -107,6 +107,7 @@ void InPlaceAbstractState::initialize()
     root->cfaHasVisited = false;
     root->cfaFoundConstants = false;
     for (size_t i = 0; i < root->valuesAtHead.numberOfArguments(); ++i) {
+        root->valuesAtTail.argument(i).clear();
         if (m_graph.m_form == SSA) {
             root->valuesAtHead.argument(i).makeHeapTop();
             continue;
@@ -158,16 +159,10 @@ void InPlaceAbstractState::initialize()
             continue;
         if (block->bytecodeBegin != m_graph.m_plan.osrEntryBytecodeIndex)
             continue;
-        for (size_t i = 0; i < m_graph.m_plan.mustHandleValues.size(); ++i) {
-            AbstractValue value;
-            value.setMostSpecific(m_graph, m_graph.m_plan.mustHandleValues[i]);
-            int operand = m_graph.m_plan.mustHandleValues.operandForIndex(i);
+        for (size_t i = 0; i < m_graph.m_mustHandleAbstractValues.size(); ++i) {
+            AbstractValue value = m_graph.m_mustHandleAbstractValues[i];
+            int operand = m_graph.m_mustHandleAbstractValues.operandForIndex(i);
             block->valuesAtHead.operand(operand).merge(value);
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-            dataLogF("    Initializing Block #%u, operand r%d, to ", blockIndex, operand);
-            block->valuesAtHead.operand(operand).dump(WTF::dataFile());
-            dataLogF("\n");
-#endif
         }
         block->cfaShouldRevisit = true;
     }
@@ -203,17 +198,11 @@ bool InPlaceAbstractState::endBasicBlock(MergeMode mergeMode)
         switch (m_graph.m_form) {
         case ThreadedCPS: {
             for (size_t argument = 0; argument < block->variablesAtTail.numberOfArguments(); ++argument) {
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-                dataLogF("        Merging state for argument %zu.\n", argument);
-#endif
                 AbstractValue& destination = block->valuesAtTail.argument(argument);
                 changed |= mergeStateAtTail(destination, m_variables.argument(argument), block->variablesAtTail.argument(argument));
             }
             
             for (size_t local = 0; local < block->variablesAtTail.numberOfLocals(); ++local) {
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-                dataLogF("        Merging state for local %zu.\n", local);
-#endif
                 AbstractValue& destination = block->valuesAtTail.local(local);
                 changed |= mergeStateAtTail(destination, m_variables.local(local), block->variablesAtTail.local(local));
             }
@@ -240,10 +229,6 @@ bool InPlaceAbstractState::endBasicBlock(MergeMode mergeMode)
     
     ASSERT(mergeMode != DontMerge || !changed);
     
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-    dataLogF("        Branch direction = %s\n", branchDirectionToString(m_branchDirection));
-#endif
-    
     reset();
     
     if (mergeMode != MergeToSuccessors)
@@ -273,16 +258,7 @@ bool InPlaceAbstractState::mergeStateAtTail(AbstractValue& destination, Abstract
         // GetLocal's result.
         
         source = inVariable;
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLogF("          Transfering ");
-        source.dump(WTF::dataFile());
-        dataLogF(" from last access due to captured variable.\n");
-#endif
     } else {
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLogF("          It's live, node @%u.\n", node->index());
-#endif
-    
         switch (node->op()) {
         case Phi:
         case SetArgument:
@@ -290,36 +266,25 @@ bool InPlaceAbstractState::mergeStateAtTail(AbstractValue& destination, Abstract
         case Flush:
             // The block transfers the value from head to tail.
             source = inVariable;
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-            dataLogF("          Transfering ");
-            source.dump(WTF::dataFile());
-            dataLogF(" from head to tail.\n");
-#endif
             break;
             
         case GetLocal:
             // The block refines the value with additional speculations.
             source = forNode(node);
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-            dataLogF("          Refining to ");
-            source.dump(WTF::dataFile());
-            dataLogF("\n");
-#endif
             break;
             
         case SetLocal:
             // The block sets the variable, and potentially refines it, both
             // before and after setting it.
-            if (node->variableAccessData()->shouldUseDoubleFormat()) {
-                // FIXME: This unnecessarily loses precision.
-                source.setType(SpecDouble);
-            } else
-                source = forNode(node->child1());
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-            dataLogF("          Setting to ");
-            source.dump(WTF::dataFile());
-            dataLogF("\n");
-#endif
+            source = forNode(node->child1());
+            if (node->variableAccessData()->flushFormat() == FlushedDouble) {
+                ASSERT(!(source.m_type & ~SpecFullNumber));
+                ASSERT(!!(source.m_type & ~SpecDouble) == !!(source.m_type & SpecMachineInt));
+                if (!(source.m_type & ~SpecDouble)) {
+                    source.merge(SpecInt52AsDouble);
+                    source.filter(SpecDouble);
+                }
+            }
             break;
         
         default:
@@ -331,9 +296,6 @@ bool InPlaceAbstractState::mergeStateAtTail(AbstractValue& destination, Abstract
     if (destination == source) {
         // Abstract execution did not change the output value of the variable, for this
         // basic block, on this iteration.
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLogF("          Not changed!\n");
-#endif
         return false;
     }
     
@@ -341,9 +303,6 @@ bool InPlaceAbstractState::mergeStateAtTail(AbstractValue& destination, Abstract
     // this variable after execution of this basic block. Update the state, and return
     // true to indicate that the fixpoint must go on!
     destination = source;
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-    dataLogF("          Changed!\n");
-#endif
     return true;
 }
 
@@ -404,23 +363,14 @@ inline bool InPlaceAbstractState::mergeToSuccessors(BasicBlock* basicBlock)
     switch (terminal->op()) {
     case Jump: {
         ASSERT(basicBlock->cfaBranchDirection == InvalidBranchDirection);
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLog("        Merging to block ", *terminal->takenBlock(), ".\n");
-#endif
         return merge(basicBlock, terminal->takenBlock());
     }
         
     case Branch: {
         ASSERT(basicBlock->cfaBranchDirection != InvalidBranchDirection);
         bool changed = false;
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLog("        Merging to block ", *terminal->takenBlock(), ".\n");
-#endif
         if (basicBlock->cfaBranchDirection != TakeFalse)
             changed |= merge(basicBlock, terminal->takenBlock());
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLog("        Merging to block ", *terminal->notTakenBlock(), ".\n");
-#endif
         if (basicBlock->cfaBranchDirection != TakeTrue)
             changed |= merge(basicBlock, terminal->notTakenBlock());
         return changed;
