@@ -24,12 +24,13 @@
 
 #include "ArrayBuffer.h"
 #include "BlockAllocator.h"
+#include "CodeBlockSet.h"
 #include "CopyVisitor.h"
-#include "DFGCodeBlocks.h"
 #include "GCIncomingRefCountedSet.h"
 #include "GCThreadSharedData.h"
 #include "HandleSet.h"
 #include "HandleStack.h"
+#include "HeapOperation.h"
 #include "JITStubRoutineSet.h"
 #include "MarkedAllocator.h"
 #include "MarkedBlock.h"
@@ -38,6 +39,7 @@
 #include "Options.h"
 #include "SlotVisitor.h"
 #include "WeakHandleOwner.h"
+#include "WriteBarrierBuffer.h"
 #include "WriteBarrierSupport.h"
 #include <wtf/HashCountedSet.h>
 #include <wtf/HashSet.h>
@@ -70,14 +72,13 @@ namespace JSC {
     typedef HashCountedSet<JSCell*> ProtectCountSet;
     typedef HashCountedSet<const char*> TypeCountSet;
 
-    enum OperationInProgress { NoOperation, Allocation, Collection };
-
     enum HeapType { SmallHeap, LargeHeap };
 
     class Heap {
         WTF_MAKE_NONCOPYABLE(Heap);
     public:
         friend class JIT;
+        friend class DFG::SpeculativeJIT;
         friend class GCThreadSharedData;
         static Heap* heap(const JSValue); // 0 for immediate values
         static Heap* heap(const JSCell*);
@@ -93,10 +94,20 @@ namespace JSC {
         static bool testAndSetMarked(const void*);
         static void setMarked(const void*);
 
+        JS_EXPORT_PRIVATE void addToRememberedSet(const JSCell*);
+        bool isInRememberedSet(const JSCell* cell) const
+        {
+            ASSERT(cell);
+            ASSERT(!Options::enableConcurrentJIT() || !isCompilationThread());
+            return MarkedBlock::blockFor(cell)->isRemembered(cell);
+        }
         static bool isWriteBarrierEnabled();
+        JS_EXPORT_PRIVATE static void writeBarrier(const JSCell*);
         static void writeBarrier(const JSCell*, JSValue);
         static void writeBarrier(const JSCell*, JSCell*);
-        static uint8_t* addressOfCardFor(JSCell*);
+
+        WriteBarrierBuffer& writeBarrierBuffer() { return m_writeBarrierBuffer; }
+        void flushWriteBarrierBuffer(JSCell*);
 
         Heap(VM*, HeapType);
         ~Heap();
@@ -111,9 +122,11 @@ namespace JSC {
         JS_EXPORT_PRIVATE void setGarbageCollectionTimerEnabled(bool);
 
         JS_EXPORT_PRIVATE IncrementalSweeper* sweeper();
+        JS_EXPORT_PRIVATE void setIncrementalSweeper(PassOwnPtr<IncrementalSweeper>);
 
         // true if collection is in progress
         inline bool isCollecting();
+        inline HeapOperation operationInProgress() { return m_operationInProgress; }
         // true if an allocation or collection is in progress
         inline bool isBusy();
         
@@ -133,9 +146,8 @@ namespace JSC {
         bool isSafeToCollect() const { return m_isSafeToCollect; }
 
         JS_EXPORT_PRIVATE void collectAllGarbage();
-        enum SweepToggle { DoNotSweep, DoSweep };
         bool shouldCollect();
-        void collect(SweepToggle);
+        void collect();
         bool collectIfNecessaryOrDefer(); // Returns true if it did collect.
 
         void reportExtraMemoryCost(size_t cost);
@@ -144,8 +156,6 @@ namespace JSC {
         JS_EXPORT_PRIVATE void protect(JSValue);
         JS_EXPORT_PRIVATE bool unprotect(JSValue); // True when the protect count drops to 0.
         
-        void jettisonDFGCodeBlock(PassRefPtr<CodeBlock>);
-
         size_t extraSize(); // extra memory usage outside of pages allocated by the heap
         JS_EXPORT_PRIVATE size_t size();
         JS_EXPORT_PRIVATE size_t capacity();
@@ -164,11 +174,13 @@ namespace JSC {
         
         template<typename Functor> typename Functor::ReturnType forEachProtectedCell(Functor&);
         template<typename Functor> typename Functor::ReturnType forEachProtectedCell();
+        template<typename Functor> inline void forEachCodeBlock(Functor&);
 
         HandleSet* handleSet() { return &m_handleSet; }
         HandleStack* handleStack() { return &m_handleStack; }
 
-        void canonicalizeCellLivenessData();
+        void willStartIterating();
+        void didFinishIterating();
         void getConservativeRegisterRoots(HashSet<JSCell*>& roots);
 
         double lastGCLength() { return m_lastGCLength; }
@@ -184,12 +196,19 @@ namespace JSC {
         const JITStubRoutineSet& jitStubRoutines() { return m_jitStubRoutines; }
         
         void addReference(JSCell*, ArrayBuffer*);
+        
+        bool isDeferred() const { return !!m_deferralDepth; }
+
+#if USE(CF)
+        template<typename T> void releaseSoon(RetainPtr<T>&&);
+#endif
 
     private:
         friend class CodeBlock;
         friend class CopiedBlock;
         friend class DeferGC;
         friend class DeferGCForAWhile;
+        friend class DelayedReleaseScope;
         friend class GCAwareJITStubRoutine;
         friend class HandleSet;
         friend class JITStubRoutine;
@@ -199,6 +218,7 @@ namespace JSC {
         friend class MarkedBlock;
         friend class CopiedSpace;
         friend class CopyVisitor;
+        friend class RecursiveAllocationScope;
         friend class SlotVisitor;
         friend class SuperRegion;
         friend class IncrementalSweeper;
@@ -216,7 +236,7 @@ namespace JSC {
         static const size_t maxExtraCost = 1024 * 1024;
         
         class FinalizerOwner : public WeakHandleOwner {
-            virtual void finalize(Handle<Unknown>, void* context);
+            virtual void finalize(Handle<Unknown>, void* context) override;
         };
 
         JS_EXPORT_PRIVATE bool isValidAllocation(size_t);
@@ -225,6 +245,7 @@ namespace JSC {
         void markRoots();
         void markProtectedObjects(HeapRootVisitor&);
         void markTempSortVectors(HeapRootVisitor&);
+        template <HeapOperation collectionType>
         void copyBackingStores();
         void harvestWeakReferences();
         void finalizeUnconditionalFinalizers();
@@ -232,6 +253,8 @@ namespace JSC {
         void zombifyDeadObjects();
         void markDeadObjects();
 
+        size_t sizeAfterCollect();
+
         JSStack& stack();
         BlockAllocator& blockAllocator();
         
@@ -244,24 +267,26 @@ namespace JSC {
         const size_t m_minBytesPerCycle;
         size_t m_sizeAfterLastCollect;
 
-        size_t m_bytesAllocatedLimit;
-        size_t m_bytesAllocated;
-        size_t m_bytesAbandoned;
+        size_t m_bytesAllocatedThisCycle;
+        size_t m_bytesAbandonedThisCycle;
+        size_t m_maxEdenSize;
+        size_t m_maxHeapSize;
+        bool m_shouldDoFullCollection;
+        size_t m_totalBytesVisited;
+        size_t m_totalBytesCopied;
         
-        OperationInProgress m_operationInProgress;
+        HeapOperation m_operationInProgress;
         BlockAllocator m_blockAllocator;
         MarkedSpace m_objectSpace;
         CopiedSpace m_storageSpace;
         GCIncomingRefCountedSet<ArrayBuffer> m_arrayBuffers;
         size_t m_extraMemoryUsage;
 
-#if ENABLE(SIMPLE_HEAP_PROFILING)
-        VTableSpectrum m_destroyedTypeCounts;
-#endif
+        HashSet<const JSCell*> m_copyingRememberedSet;
 
         ProtectCountSet m_protectedValues;
         Vector<Vector<ValueStringPair, 0, UnsafeVectorOverflow>* > m_tempSortingVectors;
-        OwnPtr<HashSet<MarkedArgumentBuffer*> > m_markListSet;
+        OwnPtr<HashSet<MarkedArgumentBuffer*>> m_markListSet;
 
         MachineThreads m_machineThreads;
         
@@ -271,12 +296,14 @@ namespace JSC {
 
         HandleSet m_handleSet;
         HandleStack m_handleStack;
-        DFGCodeBlocks m_dfgCodeBlocks;
+        CodeBlockSet m_codeBlocks;
         JITStubRoutineSet m_jitStubRoutines;
         FinalizerOwner m_finalizerOwner;
         
         bool m_isSafeToCollect;
 
+        WriteBarrierBuffer m_writeBarrierBuffer;
+
         VM* m_vm;
         double m_lastGCLength;
         double m_lastCodeDiscardTime;
@@ -305,9 +332,11 @@ namespace JSC {
 
     inline bool Heap::shouldCollect()
     {
+        if (isDeferred())
+            return false;
         if (Options::gcMaxHeapSize())
-            return m_bytesAllocated > Options::gcMaxHeapSize() && m_isSafeToCollect && m_operationInProgress == NoOperation;
-        return m_bytesAllocated > m_bytesAllocatedLimit && m_isSafeToCollect && m_operationInProgress == NoOperation;
+            return m_bytesAllocatedThisCycle > Options::gcMaxHeapSize() && m_isSafeToCollect && m_operationInProgress == NoOperation;
+        return m_bytesAllocatedThisCycle > m_maxEdenSize && m_isSafeToCollect && m_operationInProgress == NoOperation;
     }
 
     bool Heap::isBusy()
@@ -317,7 +346,7 @@ namespace JSC {
 
     bool Heap::isCollecting()
     {
-        return m_operationInProgress == Collection;
+        return m_operationInProgress == FullCollection || m_operationInProgress == EdenCollection;
     }
 
     inline Heap* Heap::heap(const JSCell* cell)
@@ -354,21 +383,33 @@ namespace JSC {
 
     inline bool Heap::isWriteBarrierEnabled()
     {
-#if ENABLE(WRITE_BARRIER_PROFILING)
+#if ENABLE(WRITE_BARRIER_PROFILING) || ENABLE(GGC)
         return true;
 #else
         return false;
 #endif
     }
 
-    inline void Heap::writeBarrier(const JSCell*, JSCell*)
+    inline void Heap::writeBarrier(const JSCell* from, JSCell* to)
     {
+#if ENABLE(WRITE_BARRIER_PROFILING)
         WriteBarrierCounters::countWriteBarrier();
+#endif
+        if (!from || !isMarked(from))
+            return;
+        if (!to || isMarked(to))
+            return;
+        Heap::heap(from)->addToRememberedSet(from);
     }
 
-    inline void Heap::writeBarrier(const JSCell*, JSValue)
+    inline void Heap::writeBarrier(const JSCell* from, JSValue to)
     {
+#if ENABLE(WRITE_BARRIER_PROFILING)
         WriteBarrierCounters::countWriteBarrier();
+#endif
+        if (!to.isCell())
+            return;
+        writeBarrier(from, to.asCell());
     }
 
     inline void Heap::reportExtraMemoryCost(size_t cost)
@@ -393,6 +434,11 @@ namespace JSC {
         return forEachProtectedCell(functor);
     }
 
+    template<typename Functor> inline void Heap::forEachCodeBlock(Functor& functor)
+    {
+        return m_codeBlocks.iterate<Functor>(functor);
+    }
+
     inline void* Heap::allocateWithNormalDestructor(size_t bytes)
     {
 #if ENABLE(ALLOCATION_LOGGING)
@@ -460,6 +506,14 @@ namespace JSC {
         return m_blockAllocator;
     }
 
+#if USE(CF)
+    template <typename T>
+    inline void Heap::releaseSoon(RetainPtr<T>&& object)
+    {
+        m_objectSpace.releaseSoon(std::move(object));
+    }
+#endif
+
 } // namespace JSC
 
 #endif // Heap_h
